@@ -28,6 +28,7 @@ pub struct Aviffy {
     width: u32,
     height: u32,
     bit_depth: u8,
+    exif: Option<Vec<u8>>,
 }
 
 /// Makes an AVIF file given encoded AV1 data (create the data with [`rav1e`](https://lib.rs/rav1e))
@@ -70,7 +71,8 @@ impl Aviffy {
             width: 0,
             height: 0,
             bit_depth: 0,
-            colr: Default::default(),
+            colr: ColrBox::default(),
+            exif: None,
         }
     }
 
@@ -153,7 +155,7 @@ impl Aviffy {
         self.make_boxes(color_av1_data, alpha_av1_data, self.width, self.height, self.bit_depth)?.write(into_output)
     }
 
-    fn make_boxes<'data>(&self, color_av1_data: &'data [u8], alpha_av1_data: Option<&'data [u8]>, width: u32, height: u32, depth_bits: u8) -> io::Result<AvifFile<'data>> {
+    fn make_boxes<'data>(&'data self, color_av1_data: &'data [u8], alpha_av1_data: Option<&'data [u8]>, width: u32, height: u32, depth_bits: u8) -> io::Result<AvifFile<'data>> {
         if ![8, 10, 12].contains(&depth_bits) {
             return Err(io::Error::new(io::ErrorKind::InvalidInput, "depth must be 8/10/12"));
         }
@@ -161,11 +163,11 @@ impl Aviffy {
         let mut image_items = ArrayVec::new();
         let mut iloc_items = ArrayVec::new();
         let mut ipma_entries = ArrayVec::new();
-        let mut data_chunks = ArrayVec::new();
         let mut irefs = ArrayVec::new();
         let mut ipco = IpcoBox::new();
         let color_image_id = 1;
         let alpha_image_id = 2;
+        let exif_id = 3;
         const ESSENTIAL_BIT: u8 = 0x80;
         let color_depth_bits = depth_bits;
         let alpha_depth_bits = depth_bits; // Sadly, the spec requires these to match.
@@ -199,16 +201,35 @@ impl Aviffy {
 
         let mut ipma = IpmaEntry {
             item_id: color_image_id,
-            prop_ids: from_array([ispe_prop, av1c_color_prop | ESSENTIAL_BIT, pixi_3])
+            prop_ids: from_array([ispe_prop, av1c_color_prop | ESSENTIAL_BIT, pixi_3]),
         };
 
         // Redundant info, already in AV1
-        if self.colr != Default::default() {
+        if self.colr != ColrBox::default() {
             let colr_color_prop = ipco.push(IpcoProp::Colr(self.colr)).ok_or(io::ErrorKind::InvalidInput)?;
             ipma.prop_ids.push(colr_color_prop);
         }
         ipma_entries.push(ipma);
 
+        if let Some(exif_data) = self.exif.as_deref() {
+            image_items.push(InfeBox {
+                id: exif_id,
+                typ: FourCC(*b"Exif"),
+                name: "",
+            });
+
+            iloc_items.push(IlocItem {
+                id: exif_id,
+                extents: [IlocExtent { data: exif_data }],
+            });
+
+            irefs.push(IrefEntryBox {
+                from_id: exif_id,
+                to_id: color_image_id,
+                typ: FourCC(*b"cdsc"),
+            });
+        }
+
         if let Some(alpha_data) = alpha_av1_data {
             image_items.push(InfeBox {
                 id: alpha_image_id,
@@ -260,34 +281,16 @@ impl Aviffy {
 
             // Use interleaved color and alpha, with alpha first.
             // Makes it possible to display partial image.
-            iloc_items.push(IlocItem {
-                id: color_image_id,
-                extents: [IlocExtent {
-                    offset: IlocOffset::Relative(alpha_data.len()),
-                    len: color_av1_data.len(),
-                }]
-                .into(),
-            });
             iloc_items.push(IlocItem {
                 id: alpha_image_id,
-                extents: [IlocExtent {
-                    offset: IlocOffset::Relative(0),
-                    len: alpha_data.len(),
-                }]
-                .into(),
-            });
-            data_chunks.push(alpha_data);
-        } else {
-            iloc_items.push(IlocItem {
-                id: color_image_id,
-                extents: [IlocExtent {
-                    offset: IlocOffset::Relative(0),
-                    len: color_av1_data.len(),
-                }]
-                .into(),
+                extents: [IlocExtent { data: alpha_data }],
             });
         }
-        data_chunks.push(color_av1_data);
+        iloc_items.push(IlocItem {
+            id: color_image_id,
+            extents: [IlocExtent { data: color_av1_data }],
+        });
+
         Ok(AvifFile {
             ftyp: FtypBox {
                 major_brand: FourCC(*b"avif"),
@@ -298,22 +301,21 @@ impl Aviffy {
                 hdlr: HdlrBox {},
                 iinf: IinfBox { items: image_items },
                 pitm: PitmBox(color_image_id),
-                iloc: IlocBox { items: iloc_items },
+                iloc: IlocBox {
+                    absolute_offset_start: None,
+                    items: iloc_items,
+                },
                 iprp: IprpBox {
                     ipco,
                     // It's not enough to define these properties,
                     // they must be assigned to the image
-                    ipma: IpmaBox {
-                        entries: ipma_entries,
-                    },
-                },
-                iref: IrefBox {
-                    entries: irefs
+                    ipma: IpmaBox { entries: ipma_entries },
                 },
+                iref: IrefBox { entries: irefs },
             },
             // Here's the actual data. If HEIF wasn't such a kitchen sink, this
             // would have been the only data this file needs.
-            mdat: MdatBox { data_chunks },
+            mdat: MdatBox,
         })
     }
 
@@ -345,6 +347,13 @@ impl Aviffy {
         self
     }
 
+    /// Set exif metadata to be included in the AVIF file as a separate item.
+    #[inline]
+    pub fn set_exif(&mut self, exif: Vec<u8>) -> &mut Self {
+        self.exif = Some(exif);
+        self
+    }
+
     /// Sets minimum required
     ///
     /// Higher bit depth may increase this
@@ -434,6 +443,20 @@ fn test_roundtrip_parse_mp4_alpha() {
     assert_eq!(&test_a[..], ctx.alpha_item_coded_data().unwrap());
 }
 
+#[test]
+fn test_roundtrip_parse_exif() {
+    let test_img = b"av12356abc";
+    let test_a = b"alpha";
+    let avif = Aviffy::new()
+        .set_exif(b"lol".to_vec())
+        .to_vec(test_img, Some(test_a), 10, 20, 8);
+
+    let ctx = mp4parse::read_avif(&mut avif.as_slice(), mp4parse::ParseStrictness::Normal).unwrap();
+
+    assert_eq!(&test_img[..], ctx.primary_item_coded_data().unwrap());
+    assert_eq!(&test_a[..], ctx.alpha_item_coded_data().unwrap());
+}
+
 #[test]
 fn test_roundtrip_parse_avif() {
     let test_img = [1, 2, 3, 4, 5, 6];
