@@ -14,12 +14,9 @@ fn main() {
     println!("<body>");
     println!("<svg height=\"800\" width=\"800\" style=\"background-color: #999\">");
     let black_path = ellipse.to_path(1e-3).to_svg();
-    println!(
-        "  <path d=\"{}\" stroke=\"black\" fill=\"none\" />",
-        black_path
-    );
+    println!("  <path d=\"{black_path}\" stroke=\"black\" fill=\"none\" />");
     let red_path = ellipse.to_path(1.0).to_svg();
-    println!("  <path d=\"{}\" stroke=\"red\" fill=\"none\" />", red_path);
+    println!("  <path d=\"{red_path}\" stroke=\"red\" fill=\"none\" />");
     println!("</svg>");
     println!("</body>");
     println!("</html>");
