@@ -393,6 +393,28 @@ impl BezPath {
         cbox.unwrap_or_default()
     }
 
+    /// Returns current position in the path, if path is not empty.
+    ///
+    /// This is different from calling [`PathEl::end_point`] on the last entry of [`BezPath::elements`]:
+    /// this method handles [`PathEl::ClosePath`],
+    /// by finding the first point of our last subpath, hence the time complexity is O(n).
+    pub fn current_position(&self) -> Option<Point> {
+        match self.0.last()? {
+            PathEl::MoveTo(p) => Some(*p),
+            PathEl::LineTo(p1) => Some(*p1),
+            PathEl::QuadTo(_, p2) => Some(*p2),
+            PathEl::CurveTo(_, _, p3) => Some(*p3),
+            PathEl::ClosePath => self
+                .elements()
+                .iter()
+                .rev()
+                .skip(1)
+                .take_while(|el| !matches!(el, PathEl::ClosePath))
+                .last()
+                .and_then(|el| el.end_point()),
+        }
+    }
+
     /// Returns a new path with the winding direction of all subpaths reversed.
     pub fn reverse_subpaths(&self) -> BezPath {
         let elements = self.elements();
@@ -651,6 +673,9 @@ pub fn flatten(
 impl Mul<PathEl> for Affine {
     type Output = PathEl;
 
+    // TODO: Inlining this leads to a huge performance benefit, perhaps the same should be done for
+    // other methods.
+    #[inline(always)]
     fn mul(self, other: PathEl) -> PathEl {
         match other {
             PathEl::MoveTo(p) => PathEl::MoveTo(self * p),
@@ -851,6 +876,22 @@ impl ParamCurve for PathSeg {
             PathSeg::Cubic(cubic) => PathSeg::Cubic(cubic.subsegment(range)),
         }
     }
+
+    fn start(&self) -> Point {
+        match *self {
+            PathSeg::Line(line) => line.start(),
+            PathSeg::Quad(quad) => quad.start(),
+            PathSeg::Cubic(cubic) => cubic.start(),
+        }
+    }
+
+    fn end(&self) -> Point {
+        match *self {
+            PathSeg::Line(line) => line.end(),
+            PathSeg::Quad(quad) => quad.end(),
+            PathSeg::Cubic(cubic) => cubic.end(),
+        }
+    }
 }
 
 impl ParamCurveArclen for PathSeg {
@@ -2019,4 +2060,53 @@ mod tests {
     fn reverse_test_helper(contour: Vec<PathEl>, expected: Vec<PathEl>) {
         assert_eq!(BezPath(contour).reverse_subpaths().0, expected);
     }
+
+    #[test]
+    fn test_rect_segments() {
+        // Ensure that `from_path_segments` does not insert spurious MoveTos in
+        // the middle of a path.
+        let x0 = 25.189500810000002;
+        let x1 = 568.18950081;
+        let y0 = -105.0;
+        let y1 = 176.0;
+        let r = Rect::from_points((x0, y0), (x1, y1));
+
+        let path0 = r.into_path(0.0);
+        assert!(path0
+            .elements()
+            .iter()
+            .skip(1)
+            .all(|el| !matches!(el, PathEl::MoveTo(_))));
+
+        let path1 = BezPath::from_path_segments(path0.segments());
+        assert!(path1
+            .elements()
+            .iter()
+            .skip(1)
+            .all(|el| !matches!(el, PathEl::MoveTo(_))));
+    }
+
+    #[test]
+    fn test_current_position() {
+        let mut path = BezPath::new();
+        assert_eq!(path.current_position(), None);
+        path.move_to((0., 0.));
+        assert_eq!(path.current_position(), Some(Point::new(0., 0.)));
+        path.line_to((10., 10.));
+        assert_eq!(path.current_position(), Some(Point::new(10., 10.)));
+        path.line_to((10., 0.));
+        assert_eq!(path.current_position(), Some(Point::new(10., 0.)));
+        path.close_path();
+        assert_eq!(path.current_position(), Some(Point::new(0., 0.)));
+
+        path.close_path();
+        assert_eq!(path.current_position(), None);
+
+        path.move_to((0., 10.));
+        assert_eq!(path.current_position(), Some(Point::new(0., 10.)));
+        path.close_path();
+        assert_eq!(path.current_position(), Some(Point::new(0., 10.)));
+        path.close_path();
+        assert_eq!(path.current_position(), None);
+    }
 }
