@@ -649,7 +649,11 @@ fn dash_impl<T: Iterator<Item = PathEl>>(
     inner: T,
     dash_offset: f64,
     dashes: &[f64],
-) -> DashIterator<T> {
+) -> DashIterator<'_, T> {
+    // ensure that offset is positive and minimal by normalization using period
+    let period = dashes.iter().sum();
+    let dash_offset = dash_offset.rem_euclid(period);
+
     let mut dash_ix = 0;
     let mut dash_remaining = dashes[dash_ix] - dash_offset;
     let mut is_active = true;
@@ -889,4 +893,13 @@ mod tests {
         let iter = segments(dash(shape.path_elements(0.), 3., &dashes));
         assert_eq!(iter.collect::<Vec<PathSeg>>(), expansion);
     }
+
+    #[test]
+    fn dash_negative_offset() {
+        let shape = Line::new((0.0, 0.0), (28.0, 0.0));
+        let dashes = [4., 2.];
+        let pos = segments(dash(shape.path_elements(0.), 60., &dashes)).collect::<Vec<PathSeg>>();
+        let neg = segments(dash(shape.path_elements(0.), -60., &dashes)).collect::<Vec<PathSeg>>();
+        assert_eq!(neg, pos);
+    }
 }
